/*
 * This file is part of Toolkit.
 *
 * Toolkit is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Toolkit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301  USA
 */
/*
available overlays:
circle.png
square.png
vertical.png
horizontal.png
*/

.toolkit-state {
    width: 32px;
    height: 10px;
    margin: 10px;
    background-color: white;
    background-image:  url(../images/state/circle.png);
    background-position: 0 0;
    background-size: 100% 100%;
    background-clip: padding-box !important;
    border-top: 3px solid #ddd;
    border-bottom: 3px solid #fff;
    border-left: 2px solid #e8e8e8;
    border-right: 2px solid #f3f3f3;
}

.toolkit-state > .toolkit-mask {
    background: black;
    opacity: 0.8;
}

.toolkit-state.toolkit-state-on > .toolkit-mask {
    opacity: 0;
}
